==> audiochain-1.1.5-src.tgz: org/audiochain/ui/cli/AliasCommand.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.cli;

import static org.audiochain.model.ObjectStore.*;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AliasCommand implements Command {
	private static final String ALIAS_PROPERTIES_FILENAME = "alias.properties";
	private Matcher m;
	private Pattern aliasPattern;
	private Collection<Command> commands = new HashSet<Command>();
	private Collection<Alias> aliases;
	private boolean aliasPatternMatches;
	private Alias matchingAlias;

	public AliasCommand(Collection<Command> commands) {
		this.commands = commands;
	}

	@Override
	public boolean matches(String command) {
		if (command.equals("alias")) {

		}
		if (aliasPattern == null) {
			aliasPattern = Pattern.compile("^\\s*alias\\s+(.*)=(.*)$");
		}
		m = aliasPattern.matcher(command);
		aliasPatternMatches = false;
		matchingAlias = null;
		if (m.matches()) {
			aliasPatternMatches = true;
			return true;
		}
		loadAliasesFromFile();
		// try to find an existing alias that matches
		for (Alias alias : aliases) {
			if (alias.matches(command)) {
				matchingAlias = alias;
				return true;
			}
		}
		return false;
	}

	@Override
	public void execute() throws Exception {
		if (aliasPatternMatches) {
			// add or remove an alias
			String keyString = m.group(1).trim();
			String commandString = m.group(2).trim();
			loadAliasesFromFile();
			if (commandString.isEmpty()) {
				// remove alias
				for (Alias alias : aliases) {
					if (alias.pattern.toString().equals(keyString)) {
						aliases.remove(alias);
						store();
						break;
					}
				}
			} else {
				Alias alias = null;

				for (Alias al : aliases) {
					if (al.pattern.toString().equals(keyString)) {
						alias = al;
					}
				}

				if (alias == null) {
					alias = new Alias();
				}
				alias.pattern = Pattern.compile(keyString);
				alias.command = commandString;

				if (alias.pattern.matcher(commandString).matches()) {
					throw new IllegalArgumentException("The alias '" + keyString + "' may not refer to itself.");
				}

				aliases.add(alias);
				store();
			}
		} else if (matchingAlias != null) {
			matchingAlias.commands = commands;
			matchingAlias.execute();
		} else {
			throw new IllegalArgumentException("Neither the alias creation pattern nor an alias matches.");
		}
	}

	@Override
	public String getDescription() {
		String linesep = System.getProperty("line.separator");
		StringBuilder sb = new StringBuilder();
		sb.append("Sets or unsets an alias <alias> for the command <command>. If <command> is kept empty, the alias will be removed.").append(linesep);
		sb.append("The <alias> can be a regular expression with groups. These matching groups can be used within the command by using $1 to $n.").append(linesep);
		sb.append("The target command may be also an alias. All build in commands are matching prior to the alias commands.").append(linesep);
		sb.append("For more information on regular expression syntax, search the web for Pattern.java").append(linesep).append(linesep);
		sb.append("Example alias definitions (abbreviated convenience aliases for audio device modification):").append(linesep);
		sb.append("  Alias for level: ^l\\s*(-?\\d*[,\\.]?\\d*)\\s*$=c level $1").append(linesep);
		sb.append("    With this new alias you can now type 'l -3,4' instead of 'c level -3.4'").append(linesep);
		sb.append("    If the value of a matching group contains a comma, this will be replaced by a dot, to become convertible into a number.").append(linesep);
		sb.append("  Alias for pan:   ^p\\s*(-?\\d*[,\\.]?\\d*)\\s*$=c pan $1").append(linesep);
		sb.append("  Alias for delay: ^d\\s*(-?\\d+)\\s*$=c delay $1");
		loadAliasesFromFile();
		if (!aliases.isEmpty()) {
			sb.append(linesep).append(linesep).append("Currently defined aliases:").append(linesep);
			sb.append(getAliasesAsString());
		}

		return sb.toString();
	}

	public String getAliasesAsString() {
		String linesep = System.getProperty("line.separator");
		StringBuilder sb = new StringBuilder();
		loadAliasesFromFile();
		boolean first = true;
		for (Alias alias : aliases) {
			if (!first) {
				sb.append(linesep);
			}
			sb.append("  " + alias.pattern + "=" + alias.command);
			first = false;
		}
		return sb.toString();
	}

	@Override
	public String getSyntax() {
		return "alias <alias>=<command>";
	}

	private void store() {
		Properties props = new Properties();
		for (Alias alias : aliases) {
			props.setProperty(alias.pattern.toString(), alias.command);
		}
		try {
			File file = createFileInSettingsDirectory(ALIAS_PROPERTIES_FILENAME);
			FileWriter fw = new FileWriter(file);
			BufferedWriter bw = new BufferedWriter(fw);
			props.store(bw, getDescription());
			bw.close();
			fw.close();
		} catch (IOException e) {
			throw new RuntimeException("Error while storing aliases: " + e.getMessage(), e);
		}
	}

	private void loadAliasesFromFile() {
		if (aliases != null) {
			return;
		}
		aliases = new LinkedHashSet<Alias>();
		Properties props = new Properties();
		try {
			File file = createFileInSettingsDirectory(ALIAS_PROPERTIES_FILENAME);
			if (!file.exists() || !file.canRead()) {
				return;
			}
			FileReader fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			props.load(br);
			br.close();
			fr.close();
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Error while loading aliases: " + e.getMessage(), e);
		} catch (IOException e) {
			throw new RuntimeException("Error while loading aliases: " + e.getMessage(), e);
		}
		aliases.clear();
		for (Entry<Object, Object> entry : props.entrySet()) {
			String key = (String) entry.getKey();
			String value = (String) entry.getValue();
			Alias alias = new Alias();
			alias.pattern = Pattern.compile(key);
			alias.command = value;
			aliases.add(alias);
		}
	}
}

class Alias implements Command {
	Collection<Command> commands;
	private Matcher m;
	Pattern pattern;
	String command;
	private boolean running;

	@Override
	public boolean matches(String command) {
		m = pattern.matcher(command);
		if (m.matches()) {
			return true;
		}
		m = null;
		return false;
	}

	@Override
	public void execute() throws Exception {
		if (running) {
			throw new IllegalArgumentException("The alias '" + pattern
					+ "' is already executing. There might be a loop in the alias definitions. Please assert, that there are no aliases defined, that references to themself directly or indirectly!");
		}
		try {
			running = true;
			String groupReplacedCommand = command;
			for (int i = 1, n = m.groupCount(); i <= n; i++) {
				String groupValue = m.group(i);
				// user convenience to have commas as dots
				groupValue = groupValue.replace(',', '.');
				groupReplacedCommand = groupReplacedCommand.replaceAll("\\$" + i, groupValue);
			}
			for (Command realCommand : commands) {
				if (realCommand.matches(groupReplacedCommand)) {
					realCommand.execute();
					return;
				}
			}
			throw new IllegalStateException("Unknown command: " + groupReplacedCommand);
		} finally {
			running = false;
		}
	}

	@Override
	public String getDescription() {
		return command;
	}

	@Override
	public String getSyntax() {
		return pattern.toString();
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/ui/cli/AliasCommand.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.ui.cli;

import static org.audiochain.model.ObjectStore.createFileInSettingsDirectory;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AliasCommand implements Command {
	private static final String ALIAS_PROPERTIES_FILENAME = "alias.properties";
	private Matcher m;
	private Pattern aliasPattern;
	private Collection<Command> commands = new HashSet<Command>();
	private Collection<Alias> aliases;
	private boolean aliasPatternMatches;
	private Alias matchingAlias;

	public AliasCommand(Collection<Command> commands) {
		this.commands = commands;
	}

	@Override
	public boolean matches(String command) {
		if (command.equals("alias")) {

		}
		if (aliasPattern == null) {
			aliasPattern = Pattern.compile("^\\s*alias\\s+(.*)=(.*)$");
		}
		m = aliasPattern.matcher(command);
		aliasPatternMatches = false;
		matchingAlias = null;
		if (m.matches()) {
			aliasPatternMatches = true;
			return true;
		}
		loadAliasesFromFile();
		// try to find an existing alias that matches
		for (Alias alias : aliases) {
			if (alias.matches(command)) {
				matchingAlias = alias;
				return true;
			}
		}
		return false;
	}

	@Override
	public void execute() throws Exception {
		if (aliasPatternMatches) {
			// add or remove an alias
			String keyString = m.group(1).trim();
			String commandString = m.group(2).trim();
			loadAliasesFromFile();
			if (commandString.isEmpty()) {
				// remove alias
				for (Alias alias : aliases) {
					if (alias.pattern.toString().equals(keyString)) {
						aliases.remove(alias);
						store();
						break;
					}
				}
			} else {
				Alias alias = null;

				for (Alias al : aliases) {
					if (al.pattern.toString().equals(keyString)) {
						alias = al;
					}
				}

				if (alias == null) {
					alias = new Alias();
				}
				alias.pattern = Pattern.compile(keyString);
				alias.command = commandString;

				if (alias.pattern.matcher(commandString).matches()) {
					throw new IllegalArgumentException("The alias '" + keyString + "' may not refer to itself.");
				}

				aliases.add(alias);
				store();
			}
		} else if (matchingAlias != null) {
			matchingAlias.commands = commands;
			matchingAlias.execute();
		} else {
			throw new IllegalArgumentException("Neither the alias creation pattern nor an alias matches.");
		}
	}

	@Override
	public String getDescription() {
		String linesep = System.getProperty("line.separator");
		StringBuilder sb = new StringBuilder();
		sb.append("Sets or unsets an alias <alias> for the command <command>. If <command> is kept empty, the alias will be removed.").append(linesep);
		sb.append("The <alias> can be a regular expression with groups. These matching groups can be used within the command by using $1 to $n.").append(linesep);
		sb.append("The target command may be also an alias. All build in commands are matching prior to the alias commands.").append(linesep);
		sb.append("For more information on regular expression syntax, search the web for Pattern.java").append(linesep).append(linesep);
		sb.append("Example alias definitions (abbreviated convenience aliases for audio device modification):").append(linesep);
		sb.append("  Alias for level: ^l\\s*(-?\\d*[,\\.]?\\d*)\\s*$=c level $1").append(linesep);
		sb.append("    With this new alias you can now type 'l -3,4' instead of 'c level -3.4'").append(linesep);
		sb.append("    If the value of a matching group contains a comma, this will be replaced by a dot, to become convertible into a number.").append(linesep);
		sb.append("  Alias for pan:   ^p\\s*(-?\\d*[,\\.]?\\d*)\\s*$=c pan $1").append(linesep);
		sb.append("  Alias for delay: ^d\\s*(-?\\d+)\\s*$=c delay $1");
		loadAliasesFromFile();
		if (!aliases.isEmpty()) {
			sb.append(linesep).append(linesep).append("Currently defined aliases:").append(linesep);
			sb.append(getAliasesAsString());
		}

		return sb.toString();
	}

	public String getAliasesAsString() {
		String linesep = System.getProperty("line.separator");
		StringBuilder sb = new StringBuilder();
		loadAliasesFromFile();
		boolean first = true;
		for (Alias alias : aliases) {
			if (!first) {
				sb.append(linesep);
			}
			sb.append("  " + alias.pattern + "=" + alias.command);
			first = false;
		}
		return sb.toString();
	}

	@Override
	public String getSyntax() {
		return "alias <alias>=<command>";
	}

	private void store() {
		Properties props = new Properties();
		for (Alias alias : aliases) {
			props.setProperty(alias.pattern.toString(), alias.command);
		}
		try {
			File file = createFileInSettingsDirectory(ALIAS_PROPERTIES_FILENAME);
			FileWriter fw = new FileWriter(file);
			BufferedWriter bw = new BufferedWriter(fw);
			props.store(bw, getDescription());
			bw.close();
			fw.close();
		} catch (IOException e) {
			throw new RuntimeException("Error while storing aliases: " + e.getMessage(), e);
		}
	}

	private void loadAliasesFromFile() {
		if (aliases != null) {
			return;
		}
		aliases = new LinkedHashSet<Alias>();
		Properties props = new Properties();
		try {
			File file = createFileInSettingsDirectory(ALIAS_PROPERTIES_FILENAME);
			if (!file.exists() || !file.canRead()) {
				return;
			}
			FileReader fr = new FileReader(file);
			BufferedReader br = new BufferedReader(fr);
			props.load(br);
			br.close();
			fr.close();
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Error while loading aliases: " + e.getMessage(), e);
		} catch (IOException e) {
			throw new RuntimeException("Error while loading aliases: " + e.getMessage(), e);
		}
		aliases.clear();
		for (Entry<Object, Object> entry : props.entrySet()) {
			String key = (String) entry.getKey();
			String value = (String) entry.getValue();
			Alias alias = new Alias();
			alias.pattern = Pattern.compile(key);
			alias.command = value;
			aliases.add(alias);
		}
	}
}

class Alias implements Command {
	Collection<Command> commands;
	private Matcher m;
	Pattern pattern;
	String command;
	private boolean running;

	@Override
	public boolean matches(String command) {
		m = pattern.matcher(command);
		if (m.matches()) {
			return true;
		}
		m = null;
		return false;
	}

	@Override
	public void execute() throws Exception {
		if (running) {
			throw new IllegalArgumentException("The alias '" + pattern
					+ "' is already executing. There might be a loop in the alias definitions. Please assert, that there are no aliases defined, that references to themself directly or indirectly!");
		}
		try {
			running = true;
			String groupReplacedCommand = command;
			for (int i = 1, n = m.groupCount(); i <= n; i++) {
				String groupValue = m.group(i);
				// user convenience to have commas as dots
				groupValue = groupValue.replace(',', '.');
				groupReplacedCommand = groupReplacedCommand.replaceAll("\\$" + i, groupValue);
			}
			for (Command realCommand : commands) {
				if (realCommand.matches(groupReplacedCommand)) {
					realCommand.execute();
					return;
				}
			}
			throw new IllegalStateException("Unknown command: " + groupReplacedCommand);
		} finally {
			running = false;
		}
	}

	@Override
	public String getDescription() {
		return command;
	}

	@Override
	public String getSyntax() {
		return pattern.toString();
	}
}
